@@ -5,16 +5,37 @@ package Template::Tiny;
 use 5.00503;
 use strict;
 
-$Template::Tiny::VERSION = '0.03';
+$Template::Tiny::VERSION = '0.05';
 
-# Parser elements
-my $left   = qr/ (?: (?: \n[ \t]* )? \[\%\- | \[\% \+? ) \s* /x;
-my $right  = qr/ \s* (?: \+? \%\] | \-\%\] (?: [ \t]*\n )? ) /x;
-my $expr   = qr/ [a-zA-Z_][\w.]*                             /x;
-my $if     = qr/ $left \s*IF\s+ ( $expr ) $right             /x;
-my $unless = qr/ $left \s*UNLESS\s+ ( $expr ) $right         /x;
-my $else   = qr/ $left \s*ELSE\s* $right                     /x;
-my $end    = qr/ $left \s*END\s* $right                      /x;
+# Evaluatable expression
+my $EXPR = qr/ [a-zA-Z_][\w.]* /xs;
+
+# Opening [% tag including whitespace chomping rules
+my $LEFT = qr/
+	(?:
+		(?: (?:^|\n) [ \t]* )? \[\%\-
+		|
+		\[\% \+?
+	) \s* /xs;
+
+# Closing %] tag including whitespace chomping rules
+my $RIGHT  = qr/
+	\s* (?:
+		\+? \%\]
+		|
+		\-\%\] (?: [ \t]* \n )?
+	) /xs;
+
+# Condition set
+my $CONDITION = qr/
+	$LEFT (IF|UNLESS) \s+ ( $EXPR ) $RIGHT
+	( .+? )
+	(?:
+		$LEFT ELSE $RIGHT
+		( .+? )
+	)?
+	$LEFT END $RIGHT
+/xs;
 
 sub new {
 	bless { }, $_[0];
@@ -28,47 +49,45 @@ sub process {
 	local $^W = 0;
 
 	$copy =~ s/
-		$if ( .+? ) $end
+		$CONDITION
 	/
-		my ($left, $right) = split $else, $2;
-		$_[0]->expression($stash, $1) ? $left : $right
+		eval {
+			$1 eq 'UNLESS'
+			xor
+			!! # Force boolification
+			$_[0]->expression($stash, $2)
+		} ? $3 : $4;
 	/gsex;
 
 	$copy =~ s/
-		$unless ( .+? ) $end
+		$LEFT ( $EXPR ) $RIGHT
 	/
-		$_[0]->expression($stash, $1) ? '' : $2
-	/gsex;
-
-	$copy =~ s/
-		$left ( $expr ) $right
-	/
-		$_[0]->expression($stash, $1)
+		eval {
+			$_[0]->expression($stash, $1)
+			. '' # Force stringification
+		}
 	/gsex;
 
 	return $copy;
 }
 
 sub expression {
-	my $value = eval {
-		my $cursor = $_[1];
-		my @path   = split /\./, $_[2];
-		foreach ( @path ) {
-			my $type = ref $cursor;
-			if ( $type eq 'ARRAY' ) {
-				return '' unless /^(?:0|[0-9]\d*)\z/;
-				$cursor = $cursor->[$_];
-			} elsif ( $type eq 'HASH' ) {
-				$cursor = $cursor->{$_};
-			} elsif ( $type ) {
-				$cursor = $cursor->$_();
-			} else {
-				return '';
-			}
+	my $cursor = $_[1];
+	my @path   = split /\./, $_[2];
+	foreach ( @path ) {
+		my $type = ref $cursor;
+		if ( $type eq 'ARRAY' ) {
+			return '' unless /^(?:0|[0-9]\d*)\z/;
+			$cursor = $cursor->[$_];
+		} elsif ( $type eq 'HASH' ) {
+			$cursor = $cursor->{$_};
+		} elsif ( $type ) {
+			$cursor = $cursor->$_();
+		} else {
+			return '';
 		}
-		return "$cursor";
-	};
-	return $value;
+	}
+	return $cursor;
 }
 
 1;
@@ -79,7 +98,7 @@ __END__
 
 =head1 NAME
 
-Template::Tiny - Template Toolkit reimplemted with as little code as possible
+Template::Tiny - Template Toolkit reimplemented in as little code as possible
 
 =head1 SYNOPSIS
 
